﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.TreeView;

namespace TreeViewDragDropImprovements
{
	public partial class MainPage : UserControl
	{
		public MainPage()
		{
			// Smaller Drag treschold means that it will be easier to start DragDrop.
			RadDragAndDropManager.DragStartThreshold = 2;

			InitializeComponent();

			// Generate some dummy data:
			listBox.ItemsSource = MyViewModel.GetItems("ListBox Item");
			leftTreeView.ItemsSource = MyViewModel.GetItems("Left Item");
			rightTreeView.ItemsSource = MyViewModel.GetItems("Right Item");

			#region Example Changing
			// The code below is used fot changing the DragDrop handlers for
			// each example.
			usageTab.SelectionChanged += new RoutedEventHandler(usageTab_SelectionChanged);
			usageTab.ItemsSource = new List<UsageExample>()
			{
				new UsageExample()
				{
					Title = "1. Default",
					Description = "No events are handled. This is how DragDrop works if you do not do anything about it.",
				},
				new UsageExample()
				{
					Title = "2. Handle Preview",
					Description = "Handles the PreviewDrop on the left TreeView. This means that dropping in the same TreeView should have no result. Dropping from right to left should remove items from the right. Moving items from left to right should 'copy' items. Please note that one and the same item should not appear in a TreeView twice because then the selection and some of its methods will stop working as expected. Sometimes it is best to create real copies of the items.",
					Load = () => leftTreeView.PreviewDragEnded += HandlePreviewDragEnded,
					Unload = () => leftTreeView.PreviewDragEnded -= HandlePreviewDragEnded,
				},
				new UsageExample()
				{
					Title = "3. Drop to List",
					Description = "Shows how the ListBox can modify the DragCue to give the user feedback. This DragCue was created by the TreeView.",
					Load = () => 
						{
							RadDragAndDropManager.AddDropQueryHandler(listBox, OnDropQuery);
							RadDragAndDropManager.AddDropInfoHandler(listBox, OnListBoxDropInfo);
						},
					Unload = () => 
						{
							RadDragAndDropManager.RemoveDropQueryHandler(listBox, OnDropQuery);
							RadDragAndDropManager.RemoveDropInfoHandler(listBox, OnListBoxDropInfo);
						}
				},
				new UsageExample()
				{
					Title = "4. Drag from List",
					Description = "Shows how the ListBox can create the TreeView DragCue so that it will have consistent DragDrop look. The DragCue is just a visual helper, the effect of the DragDrop will be the same even if no DragCue is used.",
					Load = () => 
						{
							RadDragAndDropManager.AddDragQueryHandler(listBox, OnListBoxDragQuery);
						},
					Unload = () => 
						{
							RadDragAndDropManager.RemoveDragQueryHandler(listBox, OnListBoxDragQuery);
						}
				},
				new UsageExample()
				{
				    Title = "5. Root Drop (Bug)",
				    Description = "Dropping an item before or after the root items of the TreeView on the left is forbidden. This only works correclty for DragDrop inside the same TreeView. We have not told the Source of the DragDrop that DragDrop has not actually occured and was not possible in the first place. Here we check the DropPosition of the TreeViewItem in the DropInfo handler and its Level (to determine whether it is root). The DropInfo event is only used for giving visual feedback, see next example for better implementetion.",
				    Load = () => 
				        {
				            leftTreeView.PreviewDragEnded += OnLeftTreeViewPreviewDragEnded;
				            RadDragAndDropManager.AddDropInfoHandler(leftTreeView, OnLeftTreeViewDropInfo);
				        },
				    Unload = () => 
				        {
				            leftTreeView.PreviewDragEnded -= OnLeftTreeViewPreviewDragEnded;
				            RadDragAndDropManager.RemoveDropInfoHandler(leftTreeView, OnLeftTreeViewDropInfo);
				        }
				},
				new UsageExample()
				{
				    Title = "6. Root Drop (OK)",
				    Description = "This example behvaes as the one before but it corectly handles the DropQuery event and this tells both the TreeView and the drop source that Drop there is not actually possible. Please note that the DragEndPreview event is not handled, because the TreeView will not receive a DropComplete status at all if we cancel the DragDrop like so. Please note how we register for already handled events to override the TreeView's decision. If you like, you can also sync the DragDrop directly with the DragSource and notify it in a different way that the drop is not possible. This way though you use the Dragdrop mechanics to cancel the Drop.",
				    Load = () => 
				        {
							leftTreeView.AddHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnLeftTreeViewDropQuery),  true);
				        },
				    Unload = () => 
				        {
				            leftTreeView.RemoveHandler(RadDragAndDropManager.DropQueryEvent, new EventHandler<DragDropQueryEventArgs>(OnLeftTreeViewDropQuery));
				        }
				},
				new UsageExample()
				{
				    Title = "7. Drop Inside",
				    Description = "Here the TreeView on the left allows only dropping items inside other items. This shows how the DropPosition can be altered to change where the drop occurs. Also, if you combine it withthe previos example you can see how you can enable dropping inside the root items, but not around them. It shows how the case of dropping an item in the TreeView needs to be handled seperately in this case.",
				    Load = () => 
				        {
							leftTreeView.PreviewDragEnded += DropInsidePreview;
				            RadDragAndDropManager.AddDropInfoHandler(leftTreeView, OnDropInsideTreeViewDropInfo);
				        },
				    Unload = () => 
				        {
							leftTreeView.PreviewDragEnded -= DropInsidePreview;
				            RadDragAndDropManager.RemoveDropInfoHandler(leftTreeView, OnDropInsideTreeViewDropInfo);
				        }
				}
			};
			#endregion
		}

		#region 2. Handle Preview
		private void HandlePreviewDragEnded(object sender, RadTreeViewDragEndedEventArgs e)
		{
			e.Handled = true;
		}
		#endregion

		#region 3. Drop to List
		private void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			if (e.Options.Status == DragStatus.DropDestinationQuery)
			{
				e.QueryResult = true;
				e.Handled = true;
			}
		}

		private void OnListBoxDropInfo(object sender, DragDropEventArgs e)
		{
			var cue = e.Options.DragCue as TreeViewDragCue;

			if (e.Options.Status == DragStatus.DropPossible)
			{
				if (cue != null)
				{
					cue.IsDropPossible = true;
					cue.DragActionContent = "Drop in ListBox";
					cue.DragTooltipContent = null;
				}
			}

			if (e.Options.Status == DragStatus.DropImpossible)
			{
				if (cue != null)
				{
					cue.IsDropPossible = false;
					cue.DragActionContent = null;
					cue.DragTooltipContent = null;
				}
			}

			if (e.Options.Status == DragStatus.DropComplete)
			{
				var payload = e.Options.Payload as IEnumerable;

				IList itemsSource = null;
				if (e.Options.Destination is System.Windows.Controls.ItemsControl)
				{
					itemsSource = (e.Options.Destination as System.Windows.Controls.ItemsControl).ItemsSource as IList;
				}

				foreach (var draggedItem in payload)
				{
					itemsSource.Add(draggedItem);
				}
			}
		}
		#endregion

		#region 4. Drag from List
		private void OnListBoxDragQuery(object sender, DragDropQueryEventArgs e)
		{
			if (e.Options.Status == DragStatus.DragQuery)
			{
				e.QueryResult = true;
				e.Handled = true;

				var payload = listBox.SelectedItems.Cast<object>().ToList();
				e.Options.Payload = payload;

				//Here we generate the DragCue. Technically we should generate it in the
				//DragInfo event with Status DragInProgress. This way we keep the
				//presentation and a possible business logic separate. We can do it in the
				//DragQuery event when we are not interested in this separation.
				e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
				var cue = new TreeViewDragCue();

				//Please note that we cannot set *any* source since the items may be
				//visual objects which cannot be displayed twice. This method tries to
				//find their non-visual believable representations.
				cue.SetSafeItemsSource(payload);
				cue.ItemTemplate = listBox.ItemTemplate;

				cue.IsDropPossible = false;
				//e.Options.DragCue = cue;
			}

			if (e.Options.Status == DragStatus.DropSourceQuery)
			{
				e.QueryResult = true;
				e.Handled = true;
			}
		}

		#endregion

		#region 5. Root Drop (Bug)
		void OnLeftTreeViewPreviewDragEnded(object sender, RadTreeViewDragEndedEventArgs e)
		{
			if (e.TargetDropItem != null && e.TargetDropItem.Level == 0 && e.DropPosition != DropPosition.Inside)
			{
				e.Handled = true;
			}
		}

		void OnLeftTreeViewDropInfo(object sender, DragDropEventArgs e)
		{
			var treeViewItem = e.Options.Destination as RadTreeViewItem;
			var cue = e.Options.DragCue as TreeViewDragCue;

			//We would like to forbid drag when it is in the TreeView or before or
			//after a root item.
			if (e.Options.Destination is RadTreeView
				|| (treeViewItem.Level == 0 && cue.IsDropPossible && treeViewItem.DropPosition != DropPosition.Inside))
			{
				cue.IsDropPossible = false;
				cue.DragActionContent = "Cannot drop around the roots, but you can drop in them.";
				cue.DragTooltipContent = null;
			}
		}
		#endregion

		#region 6. Root Drop (OK)
		void OnLeftTreeViewDropQuery(object sender, DragDropQueryEventArgs e)
		{
			var treeViewItem = e.Options.Destination as RadTreeViewItem;
			var cue = e.Options.DragCue as TreeViewDragCue;

			//We would like to forbid drag when it is in the TreeView or before or
			//after a root item.
			if (e.Options.Destination is RadTreeView
				|| (treeViewItem.Level == 0 && cue.IsDropPossible && treeViewItem.DropPosition != DropPosition.Inside))
			{
				//Please note how we just override the TreeView's decision about the Drop,
				//no need to set anything else, the drop will just not happen.
				e.QueryResult = false;
			}
		}
		#endregion

		#region 7. Drop Inside
		void DropInsidePreview(object sender, RadTreeViewDragEndedEventArgs e)
		{
			// If the drop is in an empty TreeView, there is no TargetDropItem, we want
			// to cancel the drop:
			if (e.TargetDropItem == null)
			{
				e.Handled = true;
				return;
			}
			//Also if the drop is before or after an item and this item is part of this TreeView, make it inside:
			if (e.DropPosition != DropPosition.Inside && e.TargetDropItem.ParentTreeView == sender)
			{
				e.TargetDropItem.DropPosition = DropPosition.Inside;
			}
		}

		void OnDropInsideTreeViewDropInfo(object sender, DragDropEventArgs e)
		{

			var treeViewItem = e.Options.Destination as RadTreeViewItem;
			var cue = e.Options.DragCue as TreeViewDragCue;
			RadTreeView treeView = null;
			//The drop destination may not be the TreeViewItem but the TreeView itself.
			//This happens if the user is dragging over an empty area of TreeView, we
			//want to drop in the last item again, so lets just find it:
			if (treeViewItem == null)
			{
				treeView = e.Options.Destination as RadTreeView;
				//We would like not to allow this kind of drop here:
				treeViewItem = treeView.ItemContainerGenerator.ContainerFromIndex(treeView.Items.Count - 1) as RadTreeViewItem;
				
				//If it is not available, we will not do anything:
				if (treeViewItem == null)
				{
					return;
				}
			}
			treeView = treeViewItem.ParentTreeView;

			if (treeViewItem != null && cue.IsDropPossible && treeViewItem.DropPosition != DropPosition.Inside)
			{
				treeViewItem.DropPosition = DropPosition.Inside;

				//But dropping inside an item may not actually be possible. We need to check.
				var isDropReallyPossible = treeViewItem.IsDropPossible(e.Options.Payload as IEnumerable);
				if (isDropReallyPossible)
				{
					treeView.HideBetweenItemsDragCue();
					cue.DragActionContent = "Drop in";
					// NOTE: Actually the line above may be written as below to accomodate any
					// localization or customization of the TreeView: 
					// cue.DragActionContent = treeView.TextDropIn
				}
				else
				{
					cue.IsDropPossible = false;
					cue.DragActionContent = null;
					cue.DragTooltipContent = null;
				}
			}
		}
		#endregion

		#region Example Changing
		void usageTab_SelectionChanged(object sender, RoutedEventArgs e)
		{
			var args = e as RadSelectionChangedEventArgs;
			foreach (var removed in args.RemovedItems.Cast<UsageExample>())
			{
				if (removed.Unload != null)
				{
					removed.Unload();
				}
			}
			foreach (var added in args.AddedItems.Cast<UsageExample>())
			{
				if (added.Load != null)
				{
					added.Load();
				}
			}
		}
		#endregion
	}
}
