﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;

namespace TreeViewDragDropImprovements
{
	public class MyViewModel : INotifyPropertyChanged
	{
		public MyViewModel()
		{
			RelatedItems = new ObservableCollection<object>();
		}

		public static IList<object> GetItems(string name)
		{
			var result = new ObservableCollection<object>();
			foreach (var num in Enumerable.Range(1, 5))
			{
				var item = new MyViewModel();
				item.DateCreated = DateTime.Today.AddDays(-num % 15);
				item.Price = num * 100 + Convert.ToDouble(num) / 100;
				item.Title = String.Format("{0} {1}", name, num);

				for (int i = 0; i < 5; i++)
				{
					var child = new MyViewModel();
					child.DateCreated = DateTime.Today.AddDays(-num % 5 - i);
					child.Price = num * 100 + Convert.ToDouble(num + i) / 100;
					child.Title = String.Format("{0} {1}'s {2}", name, num, i);

					item.RelatedItems.Add(child);
				}

				result.Add(item);
			}
			return result;
		}

		private double price;
		public double Price
		{
			get
			{
				return this.price;
			}
			set
			{
				if (this.price != value)
				{
					this.price = value;
					OnPropertyChanged("Price");
				}
			}
		}

		private DateTime dateCreated;
		public DateTime DateCreated
		{
			get
			{
				return this.dateCreated;
			}
			set
			{
				if (this.dateCreated != value)
				{
					this.dateCreated = value;
					OnPropertyChanged("DateCreated");
				}
			}
		}

		private String title;
		public String Title
		{
			get
			{
				return this.title;
			}
			set
			{
				if (this.title != value)
				{
					this.title = value;
					OnPropertyChanged("Title");
				}
			}
		}

		/// <summary>
		///     Called when the value of a property changes.
		/// </summary>
		/// <param name="propertyName">The name of the property that has changed.</param>
		protected virtual void OnPropertyChanged(String propertyName)
		{
			if (String.IsNullOrEmpty(propertyName))
			{
				return;
			}
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		public IList<object> RelatedItems
		{
			get;
			set;
		}

		/// <summary>
		///     Raised when the value of one of the properties changes.
		/// </summary>
		public event PropertyChangedEventHandler PropertyChanged;
	}
}
